unit cpu;

{$mode delphi}{$H+}

{--------------------------------------------------------------------
CPU.PAS - Epson s1c88349 CPU core emulator
Copyright (C) 2004 Paulo Marques, Paulo Costa, Pierre Delore

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--------------------------------------------------------------------}

interface

uses
  SysUtils;

type
  TCPUOperation =
    (coADD,  coADC,  coSUB,  coSBC,  coAND,  coOR,   coXOR, coCP,
     coBIT,  coINC,  coDEC,  coMLT,  coDIV,  coCPL,  coNEG,
     coLD,   coEX,   coSWAP,
     coRL,   coRLC,  coRR,   coRRC,  coSLA,  coSLL,  coSRA, coSRL,
     coPACK, coUPCK, coSEP,
     coADDw, coADCw, coSUBw, coSBCw, coCPw,  coINCw, coDECw,
     coLDw,  coEXw,
     coPUSH, coPUSHw, coPOP, coPOPw, coPUSH_ALL, coPOP_ALL, coPUSH_ALE, coPOP_ALE,
     coJRS,  coJRL,  coJP,   coDJR,  coCARS, coCARL, coCALL,
     coRET,  coRETE, coRETS, coINT,
     coNOP,  coHALT, coSLP,  coBAD, coTRAP);

const
  CPUOperationNames: array[TCPUOperation] of string =
    ('ADD',  'ADC',  'SUB',  'SBC',  'AND',  'OR',   'XOR', 'CP',
     'BIT',  'INC',  'DEC',  'MLT',  'DIV',  'CPL',  'NEG',
     'LD',   'EX',   'SWAP',
     'RL',   'RLC',  'RR',   'RRC',  'SLA',  'SLL',  'SRA', 'SRL',
     'PACK', 'UPCK', 'SEP',
     'ADDw', 'ADCw', 'SUBw', 'SBCw', 'CPw',  'INCw', 'DECw',
     'LDw',  'EXw',
     'PUSH', 'PUSHw', 'POP', 'POPw', 'PUSH_ALL', 'POP_ALL', 'PUSH_ALE', 'POP_ALE',
     'JRS',  'JRL',  'JP',   'DJR',  'CARS', 'CARL', 'CALL',
     'RET',  'RETE', 'RETS', 'INT',
     'NOP',  'HALT', 'SLP',  'BAD',  'TRAP');

type
  TCPUOperand =
    (
      // operand types who need one more byte on the opcode
      cotRefIX_Im8, cotRefIY_Im8, cotRefSP_Im8,
      cotIm8, cotRefBRIm8,

      // operand types who need two more byte on the opcode
      cotIm16, cotRefIm16,

      // operand types who don't need extra bytes on the opcode
      cotNone,
      cotA, cotB, cotH, cotL, cotBA, cotHL, cotIX, cotIY, cotSP,
      cotBR, cotNB, cotEP, cotXP, cotYP, cotSC, cotIP, cotPC,
      cotRefIX, cotRefIY, cotRefHL, cotRefIX_L, cotRefIY_L,

      cotZ, cotC, cotV, cotM,
      cotNZ, cotNC, cotNV, cotP,
      cotLT, cotLE, cotGT, cotGE,
      cotF0, cotF1, cotF2, cotF3,
      cotNF0, cotNF1, cotNF2, cotNF3
    );

const
  CPUOperandNames: array[TCPUOperand] of string =
    (
      '[IX+nn]', '[IY+nn]', '[SP+nn]',
      'nn', '[BR:nn]',

      'nnnn',  '[nnnn]',

      '',
      'A', 'B', 'H', 'L', 'BA', 'HL', 'IX', 'IY', 'SP',
      'BR', 'NB', 'EP', 'XP', 'YP', 'SC', 'IP', 'PC',
      '[IX]', '[IY]', '[HL]', '[IX+L]', '[IY+L]',

      'Z', 'C', 'V', 'M',
      'NZ', 'NC', 'NV', 'P',
      'LT', 'LE', 'GT', 'GE',
      'F0', 'F1', 'F2', 'F3',
      'NF0', 'NF1', 'NF2', 'NF3'
    );

function DisAsmInst(addr : integer; Var Str:String): integer;
function GetOpIndex(addr : integer): integer;

implementation


uses microcode,Main;


function GetOpIndex(addr : integer): integer;
var op1, op2: byte;
begin
  op1 := ReadByte(addr);

  if (op1=$CE) or (op1=$CF) then begin
    op2 := ReadByte(addr+1);
    if op1=$CE then result := op2 + $100
    else result := op2 + $200;
  end else begin
    result := op1;
  end;
end;


function DisAsmInst(addr: integer; Var Str:String): integer;
var j, index: integer;
    hex_dump: string;
begin
  Str:='';
  if AddrNames[addr]<>'' then
    Str:=AddrNames[addr] + ':';

  index := GetOpIndex(addr);

  hex_dump := '';
  for j:=0 to MicroCodeData[index].size-1 do
    hex_dump := hex_dump + inttohex(ReadByte(addr + j), 2) + ' ';

  Str := Format('%4x %-12s%s',[addr, hex_dump, MicroCodeOperandsHumanReadable(MicroCodeData[index], addr)] );

  result := Addr+MicroCodeData[index].size;
end;


end.
