unit Main;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms,Graphics, Controls,
  Dialogs,ExtCtrls, StdCtrls,IniFiles,Cpu,Microcode, Buttons;

Const
  _START_ADDR_PT=$F31A; //Start address of a periodic task in memory

type
  TWristAppInfo = Record
    Name,
    Description,
    Line1,
    Line2,
    Version_Required,
    PAR,
    CODE,
    DB,
    Password_Support,
    App_Interface : String;
  End;

  TPeriodicTaskInfo = Record
    Name,
    Description,
    Version_Required,
    CODE,
    App_Interface:String;
  End;

  TFByte = File of byte;

  { TFrmMain }

  TFrmMain = class(TForm)
    CBDispDis: TCheckBox;
    CBDispCodeBlock: TCheckBox;
    GroupBox1: TGroupBox;
    Memo: TMemo;
    Panel1: TPanel;
    BtLoad: TButton;
    BtSave: TButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    CBDbType: TComboBox;
    BtClear: TButton;
    procedure BtLoadClick(Sender: TObject);
    procedure BtSaveClick(Sender: TObject);
    procedure BtClearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    Addr_State, Addr_Common, Addr_Background : integer;
    Procedure PrintV(s:String;v:integer);
    Procedure Print(s:String);
    Function Aff1(Var f; s : String):Word;
    Function Aff2(Var f; s : String):Word;
    Function Affx(Var f; addr : integer; nb : Word):Word;
    Procedure Affs(Var f; Var j:Integer; s : String);
    Function Affsv(Var f; Var j:Integer; s : String):Integer;
    Procedure AffAbs(w:Word);
    Function Int2Str(v,nb:Integer):String;
    Function Vpp(var i:integer):integer;
    Procedure LoadAddrNames;    
  public
    { Public declarations }
    Procedure AnalyseWristapp(FName:String);
    Procedure AnalysePeriodicTask(FName:String);
  end;

var
  FrmMain: TFrmMain;
  AddrNames: array[0..$FFFF] of string;

implementation

//-- Integer to string conversion with '0' before the the number --
Function TFrmMain.Int2Str(v,nb:Integer):String;
Var s:String;
    I:Integer;
Begin
  s:=IntToStr(v);
  For I:=1 to nb-length(s) Do
    S:='0'+s;
  Int2Str:=s;
End;


//-- Display a line in the memo --
Procedure TFrmMain.Print(s:String);
Begin
  Memo.Lines.add(s);
End;


//-- Display a value in the memo (decimal/hexadecimal format and a label) --
Procedure TFrmMain.PrintV(s:String;v:integer);
Begin
  Memo.Lines.add(s+' = '+IntToStr(v)+' (0x'+IntToHex(v,4)+')');
End;


//-- Display a byte from the file in the memo (decimal/hexadecimal format and a label) --
Function TFrmMain.Aff1(Var f; s : String):Word;
Var b:byte;
    fp:longint;
Begin
  fp:=FilePos(TFByte(f));
  read(TFByte(f),b);
  Memo.Lines.add(Format('0x%s (%s) : %40s = %5d (0x%s)',[IntToHex(fp,4),Int2Str(fp,5),s,b,IntToHex(b,2)]));
  Aff1:=b;
End;


//-- Display a word from the file in the memo (decimal/hexadecimal format and a label) --
Function TFrmMain.Aff2(Var f; s : String):Word;
Var b,c:byte;
    w:word;
    fp:longint;
Begin
  fp:=FilePos(TFByte(f));
  read(TFByte(f),b);
  read(TFByte(f),c);
  w:=c*256+b;
  Memo.Lines.add(Format('0x%s (%s) : %40s = %5d (0x%s)',[IntToHex(fp,4),Int2Str(fp,5),s,w,IntToHex(w,4)]));
  Aff2:=w;
End;


//-- Display NB bytes from the file in the memo (hexadecimal) --
Function TFrmMain.Affx(Var f;addr : integer; nb : Word):Word;
Var b,c:byte;
    w,i:word;
    fp:longint;
    s:String;
Begin
  s:='';
  fp:=FilePos(TFByte(f));
  For i:=1 to nb do
  Begin
    read(TFByte(f),b);
    if (addr>=0) and (addr<=$FFFF) then mem[addr+i-1]:=b;
    s:=s+IntToHex(b,2)+' ';
    if i mod 16 = 0 then begin
      Memo.Lines.add(Format('0x%s (%s) : %-40s',[IntToHex(fp,4),Int2Str(fp,5),s]));
      s:='';
      fp:=FilePos(TFByte(f));
    end;
  End;
  if s<>'' then Memo.Lines.add(Format('0x%s (%s) : %-40s',[IntToHex(fp,4),Int2Str(fp,5),s]));
  Affx:=0;
End;


//-- Display an absolute address in the memo (decimal/hexadecimal format) --
Procedure TFrmMain.AffAbs(w:Word);
Begin
    Print(Format('                                         Absolute address = %5d (0x%s)',[w+7,IntToHex(w+7,4)]));;
End;


//-- Display a string from the file in the memo --
Procedure TFrmMain.Affs(Var f; Var j:Integer; s : String);
Var st:String;
Begin
  readln(TextFile(f),st);
  Memo.Lines.add(Format('%s %40s = %s',[Int2Str(Vpp(j),2),s,st]));
End;


//-- --
Function TFrmMain.Affsv(Var f; Var j:Integer;s : String):Integer;
Var st:String;
Begin
  readln(TextFile(f),st);
  Memo.Lines.add(Format('%s %40s = %s',[Int2Str(Vpp(j),2),s,st]));
  Affsv:=StrToInt(st);
End;


//-- --
Function TFrmMain.Vpp(var i:integer):integer;
Begin
  Vpp:=i;
  inc(i);
End;


//-- Wristapp analysis --
procedure TFrmMain.AnalyseWristapp(FName:String);
Var f:File of byte;
    ft : TextFile;
    HeaderSize, DbSize, RecordSize, i,
    NbRec, CodeSize, w : Word;
    Dir:String;
    AppIni: TIniFile;
    WristAppInfo:TWristAppInfo;
    DbType, v, j, k, l, Ad, len: Integer;
    tmps:String;
begin
  DbType:=CBDbType.ItemIndex+1;

  Dir:=ExtractFilePath(fname);

  Print('');
  Print('** Wristapp analysis **');
  Print('');
  Print('==[FILENAME: '+FName+']=======');

  AppIni := TIniFile.Create(fname);
    WristAppInfo.Name:=AppIni.ReadString('WristApp','Name','');
    WristAppInfo.Description:=AppIni.ReadString('WristApp','Description','');
    WristAppInfo.Line1:=AppIni.ReadString('WristApp','Line1','');
    WristAppInfo.Line2:=AppIni.ReadString('WristApp','Line2','');
    WristAppInfo.Version_Required:=AppIni.ReadString('WristApp','Version Required','');
    WristAppInfo.PAR:=AppIni.ReadString('WristApp','PAR','');
    WristAppInfo.CODE:=AppIni.ReadString('WristApp','CODE','');
    WristAppInfo.DB:=AppIni.ReadString('WristApp','DB','');
    WristAppInfo.Password_Support:=AppIni.ReadString('WristApp','Password Support','');
    WristAppInfo.App_Interface:=AppIni.ReadString('WristApp','App Interface','');
  AppIni.Free;

  Print('Name='+WristAppInfo.Name);
  Print('Description='+WristAppInfo.Description);
  Print('Banner line 1='+WristAppInfo.Line1);
  Print('Banner line 2='+WristAppInfo.Line2);
  Print('Version required='+WristAppInfo.Version_Required);
  Print('Parameter file='+WristAppInfo.PAR);
  Print('Code file='+WristAppInfo.CODE);
  Print('Database file='+WristAppInfo.DB);
  Print('Password Support='+WristAppInfo.Password_Support);
  Print('Application interface='+WristAppInfo.App_Interface);

  print('');

  //-- Read the builder file --

  tmps:=dir+ExtractFilename(fname);
  delete(tmps,length(tmps)-3,4);
  If FileExists(tmps+'.scr') then begin
    //-- Length of the file --
    assignfile(f,tmps+'.scr');
    system.Reset(f);
    len:=FileSize(f);
    closefile(f);


    assignfile(ft,tmps+'.scr');
    system.Reset(ft);

    Print('==[FILENAME: '+tmps+'.scr'+']=======');
    Print('File size in bytes: ' + IntToStr(len));

    j:=1;
    Affs(ft,j,'Application name');
    Affs(ft,j,'Folder name');
    Affs(ft,j,'Description');
    Affs(ft,j,'Banner Line 1');
    Affs(ft,j,'Banner Line 2');
    Affs(ft,j,'Version required');
    Affs(ft,j,'Password Support');
    Affs(ft,j,'Database Support');
    Affs(ft,j,'PC interface');
    Affs(ft,j,'Header directory');
    Affs(ft,j,'Source directory');
    Affs(ft,j,'Build directory');
    Affs(ft,j,'Include directory');
    Affs(ft,j,'Assembler directory');
    Affs(ft,j,'Numbers of type of source file');
    v:=Affsv(ft,j,'Numbers of parameter file(s)');
    For k:=1 to v do
      Affs(ft,j,'Parameter file '+IntToStr(k));
    v:=Affsv(ft,j,'Numbers of Header file(s)');
    For k:=1 to v do
      Affs(ft,j,'Header file '+IntToStr(k));
    v:=Affsv(ft,j,'Numbers of Background handler file(s)');
    For k:=1 to v do
      Affs(ft,j,'Background handler file '+IntToStr(k));
    v:=Affsv(ft,j,'Numbers of Common file(s)');
    For k:=1 to v do
      Affs(ft,j,'Common file file '+IntToStr(k));

    For l:=0 to 20 do begin
      v:=Affsv(ft,j,'Numbers of State '+IntToStr(l)+' file(s)');
      For k:=1 to v do
        Affs(ft,j,'State '+IntToStr(l)+' file '+IntToStr(k));
    End;
    Closefile(ft);
  End
  Else Print('No .scr file');
  print('');

  //-- Read the Parameter file --
  assignfile(f,dir+WristAppInfo.PAR);
  system.Reset(f);

  Print('==[FILENAME: '+dir+WristAppInfo.PAR+']=======');
  Print('File size in bytes: ' + IntToStr(FileSize(f)));

  Aff1(f,'CoreAppSystemDataOffset');
  Aff1(f,'NbResTOD');
  Aff1(f,'NbResBackup');
  Aff1(f,'NbResTimeZoneCheck');
  Aff1(f,'NbResTimer');
  Aff1(f,'NbResStopwatch');
  Aff1(f,'NbResSynchroTimer');
  Aff1(f,'LcdFlags1');
  Aff1(f,'LcdFlags2');
  Aff2(f,'HeapRequireCode');
  Aff2(f,'HeapRequireASD');
  Aff2(f,'HeapRequireADD');
  Aff1(f,'CoreACDEepromApp');
  Aff1(f,'AppType');
  Aff1(f,'AppInstance');
  Aff2(f,'ASDaddressOffset');
  Aff2(f,'ADDaddressOffset');
  Addr_State:=Aff2(f,'StateManagerAddress');
  Addr_Background:=Aff2(f,'BackgroundAddress');
  Aff2(f,'BannerMsg');
  Closefile(f);

  print('');

  Assignfile(f,dir+WristAppInfo.CODE);
  system.Reset(f);

  Print('==[FILENAME: '+dir+WristAppInfo.CODE+']=======');
  Print('File size in bytes: ' + IntToStr(FileSize(f)));

  Aff2(f,'Allocation size');
  Aff2(f,'Database Size');
  HeaderSize:=Aff1(f,'App specific Header Size');
  NbRec:=Aff2(f,'Number of records');
  If CBDispCodeBlock.Checked then begin
    If HeaderSize>2 then
    Begin
      print('-- Specific Header block ---------------------------------------');
      Affx(f,-1,HeaderSize-2);
    End;
  End;
  
  AffAbs(Aff2(f,'COMMON code block address in eeprom'));

  For i:=2 to NbRec do
    AffAbs(Aff2(f,'STATE '+IntToStr(i-2)+' code block address in eeprom'));

  Print('');
  CodeSize:=Aff2(f,'COMMON code block size');
  Aff1(f,'Reserved');
  If CBDispCodeBlock.Checked then begin
    print('-- COMMON code block -------------------------------------------');
    Affx(f,Addr_Background,CodeSize-3);
  End;
  //-- Disassemble the state --
  If CBDispDis.Checked then begin
    print('');
    Ad:=Addr_Background;
    while (Ad<Addr_Background+CodeSize-3) do begin
      Ad:=DisAsmInst(Ad,TmpS);
      print(TmpS);
    End;
  End;
  
  For i:=2 to NbRec do
  Begin
    Print('');
    CodeSize:=Aff2(f,'STATE '+IntToStr(i-2)+' code block size');
    Aff1(f,'Reserved');
    If CBDispCodeBlock.Checked then begin
      print('-- STATE '+Int2Str(i-2,2)+' code block -----------------------------------------');
      Affx(f,Addr_State,CodeSize-3);
    End;
    //-- Disassemble the state --
    If CBDispDis.Checked then begin
      print('');
      Ad:=Addr_State;
      while (Ad<Addr_State+CodeSize-3) do begin
        Ad:=DisAsmInst(Ad,TmpS);
        print(TmpS);
      End;
    End;
  End;

  Closefile(f);


  print('');

  If WristAppInfo.DB<>'' Then Begin
    Assignfile(f,dir+WristAppInfo.DB);
    system.Reset(f);

    Print('==[FILENAME: '+dir+WristAppInfo.DB+']=======');
    Print('File size in bytes: ' + IntToStr(FileSize(f)));

    Case DbType of
      1 : Begin //Sequential Database Structure
            print('-- Sequential Database type ------------------------------------');
            Aff2(f,'Allocation size');
            DbSize:=Aff2(f,'Database Size');
            HeaderSize:=Aff1(f,'App specific Header Size');
            print('-- Specific Header block ---------------------------------------');
            Affx(f,-1,HeaderSize-2);
            print('-- Record #0 ---------------------------------------------------');
            Affx(f,-1,DbSize-HeaderSize-4);
          End;
      2 : Begin //Fixed-sized random database structure
            print('-- Fixed-sized random database type ----------------------------');
            Aff2(f,'Allocation size');
            DbSize:=Aff2(f,'Database Size');
            HeaderSize:=Aff1(f,'App specific Header Size');
            print('-- Specific Header block ---------------------------------------');
            NbRec:=Aff2(f,'Number of records');
            RecordSize:=Aff1(f,'Record Size');
            Affx(f,-1,HeaderSize-2-1);
            For i:=1 to NbRec do
            Begin
              print('-- Record Data '+Int2Str(i,3)+' ---------------------------------------------');
              Affx(f,-1,RecordSize);
            End;
          End;
      3 : Begin //Variable-sized random database structure

          End;
      4 : Begin //Link-list database structure

          End;
    End; //Case

    Closefile(f);
  End;
end;


//-- Periodic task analysis --
procedure TFrmMain.AnalysePeriodicTask(FName:String);
Var f:File of byte;
    HeaderSize, DbSize, RecordSize, i,
    NbRec, CodeSize, w : Word;
    Dir:String;
    AppIni: TIniFile;
    WristAppInfo:TWristAppInfo;
    PeriodicTaskInfo:TPeriodicTaskInfo;

    ft : TextFile;
    v,j, k, l, Ad, len: Integer;
    tmps:String;
begin
  Dir:=ExtractFilePath(fname);

  Print('');
  Print('** Periodic task analysis **');
  Print('');
  Print('==[FILENAME: '+FName+']=======');

  AppIni := TIniFile.Create(fname);

  AppIni := TIniFile.Create(fname);
  PeriodicTaskInfo.Name:=AppIni.ReadString('Periodic Task','Name','');
  PeriodicTaskInfo.Description:=AppIni.ReadString('Periodic Task','Description','');
  PeriodicTaskInfo.Version_Required:=AppIni.ReadString('Periodic Task','Version Required','');
  PeriodicTaskInfo.CODE:=AppIni.ReadString('Periodic Task','CODE','');
  PeriodicTaskInfo.App_Interface:=AppIni.ReadString('Periodic Task','App Interface','');
  AppIni.Free;

  Print('Name='+PeriodicTaskInfo.Name);
  Print('Description='+PeriodicTaskInfo.Description);
  Print('Version required='+PeriodicTaskInfo.Version_Required);
  Print('Code file='+PeriodicTaskInfo.CODE);
  Print('Application interface='+WristAppInfo.App_Interface);

  print('');


  //-- Read the builder file --

  tmps:=dir+ExtractFilename(fname);
  delete(tmps,length(tmps)-3,4);
  If FileExists(tmps+'.psc') then begin
    //-- Length of the file --
    assignfile(f,tmps+'.psc');
    system.Reset(f);
    len:=FileSize(f);
    closefile(f);
    
    //-- Analyse the file --
    assignfile(ft,tmps+'.psc');
    system.Reset(ft);

    Print('==[FILENAME: '+tmps+'.psc'+']=======');
    Print('File size in bytes: ' + IntToStr(len));

    j:=1;
    Affs(ft,j,'Application name');
    Affs(ft,j,'Folder name');
    Affs(ft,j,'Description');
    Affs(ft,j,'Version required');
    Affs(ft,j,'PC interface');
    Affs(ft,j,'Allow minute task execution');
    Affs(ft,j,'Allow hour task execution');
    Affs(ft,j,'Allow daily task execution');
    Affs(ft,j,'Header directory');
    Affs(ft,j,'Source directory');
    Affs(ft,j,'Build directory');
    Affs(ft,j,'Include directory');
    Affs(ft,j,'Assembler directory');

    Affs(ft,j,'Numbers of type of source file');

    v:=Affsv(ft,j,'Numbers of minute task file(s)');
    For k:=1 to v do
      Affs(ft,j,'Minute task file '+IntToStr(k));

    v:=Affsv(ft,j,'Numbers of hour task file(s)');
    For k:=1 to v do
      Affs(ft,j,'Hour task file '+IntToStr(k));

    v:=Affsv(ft,j,'Numbers of day task file(s)');
    For k:=1 to v do
      Affs(ft,j,'Day task file '+IntToStr(k));

    Closefile(ft);
  end
  else
    Print('No .psc file !');

  print('');


  //-- Analyse code ---
  Assignfile(f,dir+PeriodicTaskInfo.CODE);
  system.Reset(f);

  Print('==[FILENAME: '+dir+PeriodicTaskInfo.CODE+']=======');
  Print('File size in bytes: ' + IntToStr(FileSize(f)));

  Aff2(f,'Allocation size');
  Aff2(f,'Database Size');
  HeaderSize:=Aff1(f,'App specific Header Size');
  NbRec:=Aff2(f,'Number of records');
  print('-- Specific Header block ---------------------------------------');
  Aff1(f,'Periodic task mask');

  AffAbs(Aff2(f,'SECOND code block address in eeprom')+1);
  AffAbs(Aff2(f,'MINUTE code block address in eeprom')+1);
  AffAbs(Aff2(f,'HOUR code block address in eeprom')+1);
  AffAbs(Aff2(f,'DAY code block address in eeprom')+1);

  Print('');
  CodeSize:=Aff2(f,'SECOND code block size');
  Aff1(f,'Reserved');
  If CBDispCodeBlock.Checked then begin
    print('-- SECOND code block -------------------------------------------');
    Affx(f,_START_ADDR_PT,CodeSize-3);
  End;
  //-- Disassemble the state --
  If CBDispDis.Checked then begin
    print('');
    Ad:=_START_ADDR_PT;
    while (Ad<_START_ADDR_PT+integer(CodeSize)-3) do begin
      Ad:=DisAsmInst(Ad,TmpS);
      print(TmpS);
    End;
  End;
  
  Print('');
  CodeSize:=Aff2(f,'MINUTE code block size');
  Aff1(f,'Reserved');
  If CBDispCodeBlock.Checked then begin
    print('-- MINUTE code block -------------------------------------------');
    Affx(f,_START_ADDR_PT,CodeSize-3);
  End;
  //-- Disassemble the state --
  If CBDispDis.Checked then begin
    print('');
    Ad:=_START_ADDR_PT;
    while (Ad<_START_ADDR_PT+integer(CodeSize)-3) do begin
      Ad:=DisAsmInst(Ad,TmpS);
      print(TmpS);
    End;
  End;
  
  Print('');
  CodeSize:=Aff2(f,'HOUR code block size');
  Aff1(f,'Reserved');
  If CBDispCodeBlock.Checked then begin
    print('-- HOUR code block ---------------------------------------------');
    Affx(f,_START_ADDR_PT,CodeSize-3);
  End;
  //-- Disassemble the state --
  If CBDispDis.Checked then begin
    print('');
    Ad:=_START_ADDR_PT;
    while (Ad<_START_ADDR_PT+integer(CodeSize)-3) do begin
      Ad:=DisAsmInst(Ad,TmpS);
      print(TmpS);
    End;
  End;
  
  Print('');
  CodeSize:=Aff2(f,'DAY code block size');
  Aff1(f,'Reserved');
  If CBDispCodeBlock.Checked then begin
    print('-- DAY code block ----------------------------------------------');
    Affx(f,_START_ADDR_PT,CodeSize-3);
  End;
  //-- Disassemble the state --
  If CBDispDis.Checked then begin
    print('');
    Ad:=_START_ADDR_PT;
    while (Ad<_START_ADDR_PT+integer(CodeSize)-3) do begin
      Ad:=DisAsmInst(Ad,TmpS);
      print(TmpS);
    End;
  End;
  
  Closefile(f);

end;


//-- Called if the button LOAD is pressed --
procedure TFrmMain.BtLoadClick(Sender: TObject);
begin
  OpenDialog.FileName:='';
  OpenDialog.DefaultExt:='app';
  OpenDialog.InitialDir:=''; //DirWristApp;
  OpenDialog.Filter:='Wristapp file (*.app)|*.app|Periodic task (*.tsk)|*.tsk';
  if OpenDialog.Execute then begin
    If Uppercase(ExtractFileExt(OpenDialog.FileName))='.APP' Then AnalyseWristapp(OpenDialog.FileName);
    If Uppercase(ExtractFileExt(OpenDialog.FileName))='.TSK' Then AnalysePeriodicTask(OpenDialog.FileName);
  End;
end;


//-- Called if the button SAVE is pressed --
procedure TFrmMain.BtSaveClick(Sender: TObject);
begin
  if SaveDialog.Execute then Begin
    Memo.Lines.SaveToFile(SaveDialog.FileName);
  End;
end;


//-- Called if the button CLEAR is pressed --
procedure TFrmMain.BtClearClick(Sender: TObject);
begin
  Memo.lines.Clear;
end;


//-- Called at the form creation --
procedure TFrmMain.FormCreate(Sender: TObject);
Var i:integer;
begin
  //-- Initialize the memory --
  For i:=0 to $FFFF do
    Mem[i]:=0;
  InitMicroCode; //-- Initialize the microcode

  Addr_State:=0;
  Addr_Common:=0;
  Addr_Background:=0;
  LoadAddrNames;
end;


//-- Load the Symbol file --
procedure TFrmMain.LoadAddrNames;
var sl: TStringList;
    i, p, addr: integer;
begin
  sl := TStringList.Create;
  try
    sl.LoadFromFile('labels.cfg');
    for i:=0 to sl.Count-1 do begin
      p := Pos('=', sl[i]);
      if p<>0 then begin
        addr := strtointdef('$' + copy(sl[i], p+1, 5),-1);
        if (addr>=0) and (addr<$10000) then begin
          AddrNames[addr] := trim(copy(sl[i],1,p-1));
        end;
      end;
    end;
  finally
    sl.Free;
  end;
end;


initialization
  {$I main.lrs}
end.
