unit microcode;

{$mode delphi}{$H+}

{--------------------------------------------------------------------
MICROCODE.PAS - opcode definition and parsing for the epson s1c88349 cpu
Copyright (C) 2004 Paulo Marques, Paulo Costa

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--------------------------------------------------------------------}

interface

uses cpu;

type
  TCPUMicroCode = record
    // size of the instruction in bytes
    size: integer;
    // cycles the instruction takes to execute (+2 when a conditional call actually suceeds)
    cycles: integer;
    // type of operation (ADD, ADC, SUB, etc.)
    op: TCPUOperation;
    // type of operands
    dest, src: TCPUOperand;
    // offset from the address of the opcode where the imediate values
    // are to be read
    d_imm_off, s_imm_off: integer;
  end;

var
  MicroCodeData: array[0..$2FF] of TCPUMicroCode = (
    (size: 1; cycles: 2; op:coADD   ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 2; op:coADD   ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 2; op:coADD   ; dest: cotA        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coADD   ; dest: cotA        ; src: cotRefHL    ),
    (size: 2; cycles: 3; op:coADD   ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 3; cycles: 4; op:coADD   ; dest: cotA        ; src: cotRefIm16  ),
    (size: 1; cycles: 2; op:coADD   ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coADD   ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coADC   ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 2; op:coADC   ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 2; op:coADC   ; dest: cotA        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coADC   ; dest: cotA        ; src: cotRefHL    ),
    (size: 2; cycles: 3; op:coADC   ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 3; cycles: 4; op:coADC   ; dest: cotA        ; src: cotRefIm16  ),
    (size: 1; cycles: 2; op:coADC   ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coADC   ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coSUB   ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 2; op:coSUB   ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 2; op:coSUB   ; dest: cotA        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coSUB   ; dest: cotA        ; src: cotRefHL    ),
    (size: 2; cycles: 3; op:coSUB   ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 3; cycles: 4; op:coSUB   ; dest: cotA        ; src: cotRefIm16  ),
    (size: 1; cycles: 2; op:coSUB   ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coSUB   ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coSBC   ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 2; op:coSBC   ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 2; op:coSBC   ; dest: cotA        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coSBC   ; dest: cotA        ; src: cotRefHL    ),
    (size: 2; cycles: 3; op:coSBC   ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 3; cycles: 4; op:coSBC   ; dest: cotA        ; src: cotRefIm16  ),
    (size: 1; cycles: 2; op:coSBC   ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coSBC   ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coAND   ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 2; op:coAND   ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 2; op:coAND   ; dest: cotA        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coAND   ; dest: cotA        ; src: cotRefHL    ),
    (size: 2; cycles: 3; op:coAND   ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 3; cycles: 4; op:coAND   ; dest: cotA        ; src: cotRefIm16  ),
    (size: 1; cycles: 2; op:coAND   ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coAND   ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coOR    ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 2; op:coOR    ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 2; op:coOR    ; dest: cotA        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coOR    ; dest: cotA        ; src: cotRefHL    ),
    (size: 2; cycles: 3; op:coOR    ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 3; cycles: 4; op:coOR    ; dest: cotA        ; src: cotRefIm16  ),
    (size: 1; cycles: 2; op:coOR    ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coOR    ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coCP    ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 2; op:coCP    ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 2; op:coCP    ; dest: cotA        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coCP    ; dest: cotA        ; src: cotRefHL    ),
    (size: 2; cycles: 3; op:coCP    ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 3; cycles: 4; op:coCP    ; dest: cotA        ; src: cotRefIm16  ),
    (size: 1; cycles: 2; op:coCP    ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coCP    ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coXOR   ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 2; op:coXOR   ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 2; op:coXOR   ; dest: cotA        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coXOR   ; dest: cotA        ; src: cotRefHL    ),
    (size: 2; cycles: 3; op:coXOR   ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 3; cycles: 4; op:coXOR   ; dest: cotA        ; src: cotRefIm16  ),
    (size: 1; cycles: 2; op:coXOR   ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coXOR   ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotA        ; src: cotA        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotA        ; src: cotB        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotA        ; src: cotL        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotA        ; src: cotH        ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotA        ; src: cotRefBRIm8 ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotA        ; src: cotRefHL    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotA        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotA        ; src: cotRefIY    ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotB        ; src: cotA        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotB        ; src: cotB        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotB        ; src: cotL        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotB        ; src: cotH        ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotB        ; src: cotRefBRIm8 ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotB        ; src: cotRefHL    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotB        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotB        ; src: cotRefIY    ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotL        ; src: cotA        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotL        ; src: cotB        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotL        ; src: cotL        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotL        ; src: cotH        ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotL        ; src: cotRefBRIm8 ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotL        ; src: cotRefHL    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotL        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotL        ; src: cotRefIY    ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotH        ; src: cotA        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotH        ; src: cotB        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotH        ; src: cotL        ),
    (size: 1; cycles: 1; op:coLD    ; dest: cotH        ; src: cotH        ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotH        ; src: cotRefBRIm8 ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotH        ; src: cotRefHL    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotH        ; src: cotRefIX    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotH        ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefIX    ; src: cotA        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefIX    ; src: cotB        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefIX    ; src: cotL        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefIX    ; src: cotH        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIX    ; src: cotRefBRIm8 ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefIX    ; src: cotRefHL    ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefIX    ; src: cotRefIX    ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefIX    ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefHL    ; src: cotA        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefHL    ; src: cotB        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefHL    ; src: cotL        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefHL    ; src: cotH        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefHL    ; src: cotRefBRIm8 ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefHL    ; src: cotRefHL    ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefIY    ; src: cotA        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefIY    ; src: cotB        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefIY    ; src: cotL        ),
    (size: 1; cycles: 2; op:coLD    ; dest: cotRefIY    ; src: cotH        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIY    ; src: cotRefBRIm8 ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefIY    ; src: cotRefHL    ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefIY    ; src: cotRefIX    ),
    (size: 1; cycles: 3; op:coLD    ; dest: cotRefIY    ; src: cotRefIY    ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotRefBRIm8 ; src: cotA        ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotRefBRIm8 ; src: cotB        ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotRefBRIm8 ; src: cotL        ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotRefBRIm8 ; src: cotH        ),
    (size: 1; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefBRIm8 ; src: cotRefHL    ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefBRIm8 ; src: cotRefIX    ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefBRIm8 ; src: cotRefIY    ),
    (size: 1; cycles: 2; op:coINC   ; dest: cotA        ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINC   ; dest: cotB        ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINC   ; dest: cotL        ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINC   ; dest: cotH        ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINC   ; dest: cotBR       ; src: cotNone     ),
    (size: 2; cycles: 4; op:coINC   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 1; cycles: 3; op:coINC   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINCw  ; dest: cotSP       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDEC   ; dest: cotA        ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDEC   ; dest: cotB        ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDEC   ; dest: cotL        ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDEC   ; dest: cotH        ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDEC   ; dest: cotBR       ; src: cotNone     ),
    (size: 2; cycles: 4; op:coDEC   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 1; cycles: 3; op:coDEC   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDECw  ; dest: cotSP       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINCw  ; dest: cotBA       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINCw  ; dest: cotHL       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINCw  ; dest: cotIX       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coINCw  ; dest: cotIY       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coBIT   ; dest: cotA        ; src: cotB        ),
    (size: 2; cycles: 3; op:coBIT   ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coBIT   ; dest: cotA        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coBIT   ; dest: cotB        ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coDECw  ; dest: cotBA       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDECw  ; dest: cotHL       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDECw  ; dest: cotIX       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coDECw  ; dest: cotIY       ; src: cotNone     ),
    (size: 2; cycles: 2; op:coAND   ; dest: cotSC       ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coOR    ; dest: cotSC       ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coXOR   ; dest: cotSC       ; src: cotIm8      ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotSC       ; src: cotIm8      ),
    (size: 1; cycles: 4; op:coPUSHw ; dest: cotBA       ; src: cotNone     ),
    (size: 1; cycles: 4; op:coPUSHw ; dest: cotHL       ; src: cotNone     ),
    (size: 1; cycles: 4; op:coPUSHw ; dest: cotIX       ; src: cotNone     ),
    (size: 1; cycles: 4; op:coPUSHw ; dest: cotIY       ; src: cotNone     ),
    (size: 1; cycles: 3; op:coPUSH  ; dest: cotBR       ; src: cotNone     ),
    (size: 1; cycles: 3; op:coPUSH  ; dest: cotEP       ; src: cotNone     ),
    (size: 1; cycles: 4; op:coPUSHw ; dest: cotIP       ; src: cotNone     ),
    (size: 1; cycles: 3; op:coPUSH  ; dest: cotSC       ; src: cotNone     ),
    (size: 1; cycles: 3; op:coPOPw  ; dest: cotBA       ; src: cotNone     ),
    (size: 1; cycles: 3; op:coPOPw  ; dest: cotHL       ; src: cotNone     ),
    (size: 1; cycles: 3; op:coPOPw  ; dest: cotIX       ; src: cotNone     ),
    (size: 1; cycles: 3; op:coPOPw  ; dest: cotIY       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coPOP   ; dest: cotBR       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coPOP   ; dest: cotEP       ; src: cotNone     ),
    (size: 1; cycles: 3; op:coPOPw  ; dest: cotIP       ; src: cotNone     ),
    (size: 1; cycles: 2; op:coPOP   ; dest: cotSC       ; src: cotNone     ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotA        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotB        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotL        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotH        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotBR       ; src: cotIm8      ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotRefIX    ; src: cotIm8      ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotRefIY    ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coLDw   ; dest: cotBA       ; src: cotRefIm16  ),
    (size: 3; cycles: 3; op:coLDw   ; dest: cotHL       ; src: cotRefIm16  ),
    (size: 3; cycles: 3; op:coLDw   ; dest: cotIX       ; src: cotRefIm16  ),
    (size: 3; cycles: 3; op:coLDw   ; dest: cotIY       ; src: cotRefIm16  ),
    (size: 3; cycles: 5; op:coLDw   ; dest: cotRefIm16  ; src: cotBA       ),
    (size: 3; cycles: 5; op:coLDw   ; dest: cotRefIm16  ; src: cotHL       ),
    (size: 3; cycles: 5; op:coLDw   ; dest: cotRefIm16  ; src: cotIX       ),
    (size: 3; cycles: 5; op:coLDw   ; dest: cotRefIm16  ; src: cotIY       ),
    (size: 3; cycles: 3; op:coADDw  ; dest: cotBA       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coADDw  ; dest: cotHL       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coADDw  ; dest: cotIX       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coADDw  ; dest: cotIY       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coLDw   ; dest: cotBA       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coLDw   ; dest: cotHL       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coLDw   ; dest: cotIX       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coLDw   ; dest: cotIY       ; src: cotIm16     ),
    (size: 1; cycles: 3; op:coEXw   ; dest: cotBA       ; src: cotHL       ),
    (size: 1; cycles: 3; op:coEXw   ; dest: cotBA       ; src: cotIX       ),
    (size: 1; cycles: 3; op:coEXw   ; dest: cotBA       ; src: cotIY       ),
    (size: 1; cycles: 3; op:coEXw   ; dest: cotBA       ; src: cotSP       ),
    (size: 1; cycles: 2; op:coEX    ; dest: cotA        ; src: cotB        ),
    (size: 1; cycles: 3; op:coEX    ; dest: cotA        ; src: cotRefHL    ),
    (size: 1; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 1; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 3; op:coSUBw  ; dest: cotBA       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coSUBw  ; dest: cotHL       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coSUBw  ; dest: cotIX       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coSUBw  ; dest: cotIY       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coCPw   ; dest: cotBA       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coCPw   ; dest: cotHL       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coCPw   ; dest: cotIX       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coCPw   ; dest: cotIY       ; src: cotIm16     ),
    (size: 3; cycles: 5; op:coAND   ; dest: cotRefBRIm8 ; src: cotIm8      ),
    (size: 3; cycles: 5; op:coOR    ; dest: cotRefBRIm8 ; src: cotIm8      ),
    (size: 3; cycles: 5; op:coXOR   ; dest: cotRefBRIm8 ; src: cotIm8      ),
    (size: 3; cycles: 4; op:coCP    ; dest: cotRefBRIm8 ; src: cotIm8      ),
    (size: 3; cycles: 4; op:coBIT   ; dest: cotRefBRIm8 ; src: cotIm8      ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefBRIm8 ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coPACK  ; dest: cotNone     ; src: cotNone     ),
    (size: 1; cycles: 2; op:coUPCK  ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coCARS  ; dest: cotC        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coCARS  ; dest: cotNC       ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coCARS  ; dest: cotZ        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coCARS  ; dest: cotNZ       ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coJRS   ; dest: cotC        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coJRS   ; dest: cotNC       ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coJRS   ; dest: cotZ        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coJRS   ; dest: cotNZ       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARL  ; dest: cotC        ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coCARL  ; dest: cotNC       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coCARL  ; dest: cotZ        ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coCARL  ; dest: cotNZ       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coJRL   ; dest: cotC        ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coJRL   ; dest: cotNC       ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coJRL   ; dest: cotZ        ; src: cotIm16     ),
    (size: 3; cycles: 3; op:coJRL   ; dest: cotNZ       ; src: cotIm16     ),
    (size: 2; cycles: 4; op:coCARS  ; dest: cotIm8      ; src: cotNone     ),
    (size: 2; cycles: 2; op:coJRS   ; dest: cotIm8      ; src: cotNone     ),
    (size: 3; cycles: 5; op:coCARL  ; dest: cotIm16     ; src: cotNone     ),
    (size: 3; cycles: 3; op:coJRL   ; dest: cotIm16     ; src: cotNone     ),
    (size: 1; cycles: 2; op:coJP    ; dest: cotHL       ; src: cotNone     ),
    (size: 2; cycles: 4; op:coDJR   ; dest: cotNZ       ; src: cotIm8      ),
    (size: 1; cycles: 2; op:coSWAP  ; dest: cotA        ; src: cotNone     ),
    (size: 1; cycles: 3; op:coSWAP  ; dest: cotRefHL    ; src: cotNone     ),
    (size: 1; cycles: 3; op:coRET   ; dest: cotNone     ; src: cotNone     ),
    (size: 1; cycles: 4; op:coRETE  ; dest: cotNone     ; src: cotNone     ),
    (size: 1; cycles: 5; op:coRETS  ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 7; op:coCALL  ; dest: cotRefIm16  ; src: cotNone     ),
    (size: 2; cycles: 7; op:coINT   ; dest: cotIm8      ; src: cotNone     ),
    (size: 2; cycles: 4; op:coJP    ; dest: cotIm8      ; src: cotNone     ),
    (size: 1; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 1; cycles: 2; op:coNOP   ; dest: cotNone     ; src: cotNone     ),

    // $CE expanded opcodes
    (size: 3; cycles: 4; op:coADD   ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coADD   ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coADD   ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coADD   ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 2; cycles: 4; op:coADD   ; dest: cotRefHL    ; src: cotA        ),
    (size: 3; cycles: 5; op:coADD   ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 5; op:coADD   ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coADD   ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 3; cycles: 4; op:coADC   ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coADC   ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coADC   ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coADC   ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 2; cycles: 4; op:coADC   ; dest: cotRefHL    ; src: cotA        ),
    (size: 3; cycles: 5; op:coADC   ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 5; op:coADC   ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coADC   ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 3; cycles: 4; op:coSUB   ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coSUB   ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coSUB   ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coSUB   ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 2; cycles: 4; op:coSUB   ; dest: cotRefHL    ; src: cotA        ),
    (size: 3; cycles: 5; op:coSUB   ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 5; op:coSUB   ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coSUB   ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 3; cycles: 4; op:coSBC   ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coSBC   ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coSBC   ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coSBC   ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 2; cycles: 4; op:coSBC   ; dest: cotRefHL    ; src: cotA        ),
    (size: 3; cycles: 5; op:coSBC   ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 5; op:coSBC   ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coSBC   ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 3; cycles: 4; op:coAND   ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coAND   ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coAND   ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coAND   ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 2; cycles: 4; op:coAND   ; dest: cotRefHL    ; src: cotA        ),
    (size: 3; cycles: 5; op:coAND   ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 5; op:coAND   ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coAND   ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 3; cycles: 4; op:coOR    ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coOR    ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coOR    ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coOR    ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 2; cycles: 4; op:coOR    ; dest: cotRefHL    ; src: cotA        ),
    (size: 3; cycles: 5; op:coOR    ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 5; op:coOR    ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coOR    ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 3; cycles: 4; op:coCP    ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coCP    ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coCP    ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coCP    ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 2; cycles: 3; op:coCP    ; dest: cotRefHL    ; src: cotA        ),
    (size: 3; cycles: 4; op:coCP    ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 4; op:coCP    ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 2; cycles: 4; op:coCP    ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 3; cycles: 4; op:coXOR   ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coXOR   ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coXOR   ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coXOR   ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 2; cycles: 4; op:coXOR   ; dest: cotRefHL    ; src: cotA        ),
    (size: 3; cycles: 5; op:coXOR   ; dest: cotRefHL    ; src: cotIm8      ),
    (size: 2; cycles: 5; op:coXOR   ; dest: cotRefHL    ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coXOR   ; dest: cotRefHL    ; src: cotRefIY    ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotA        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coLD    ; dest: cotA        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coLD    ; dest: cotA        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotA        ; src: cotRefIY_L  ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefIX_Im8; src: cotA        ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefIY_Im8; src: cotA        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIX_L  ; src: cotA        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIY_L  ; src: cotA        ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotB        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coLD    ; dest: cotB        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coLD    ; dest: cotB        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotB        ; src: cotRefIY_L  ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefIX_Im8; src: cotB        ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefIY_Im8; src: cotB        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIX_L  ; src: cotB        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIY_L  ; src: cotB        ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotL        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coLD    ; dest: cotL        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coLD    ; dest: cotL        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotL        ; src: cotRefIY_L  ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefIX_Im8; src: cotL        ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefIY_Im8; src: cotL        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIX_L  ; src: cotL        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIY_L  ; src: cotL        ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotH        ; src: cotRefIX_Im8),
    (size: 3; cycles: 4; op:coLD    ; dest: cotH        ; src: cotRefIY_Im8),
    (size: 2; cycles: 4; op:coLD    ; dest: cotH        ; src: cotRefIX_L  ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotH        ; src: cotRefIY_L  ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefIX_Im8; src: cotH        ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotRefIY_Im8; src: cotH        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIX_L  ; src: cotH        ),
    (size: 2; cycles: 4; op:coLD    ; dest: cotRefIY_L  ; src: cotH        ),
    (size: 3; cycles: 5; op:coLD    ; dest: cotRefHL    ; src: cotRefIX_Im8),
    (size: 3; cycles: 5; op:coLD    ; dest: cotRefHL    ; src: cotRefIY_Im8),
    (size: 2; cycles: 5; op:coLD    ; dest: cotRefHL    ; src: cotRefIX_L  ),
    (size: 2; cycles: 5; op:coLD    ; dest: cotRefHL    ; src: cotRefIY_L  ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 5; op:coLD    ; dest: cotRefIX    ; src: cotRefIX_Im8),
    (size: 3; cycles: 5; op:coLD    ; dest: cotRefIX    ; src: cotRefIY_Im8),
    (size: 2; cycles: 5; op:coLD    ; dest: cotRefIX    ; src: cotRefIX_L  ),
    (size: 2; cycles: 5; op:coLD    ; dest: cotRefIX    ; src: cotRefIY_L  ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 5; op:coLD    ; dest: cotRefIY    ; src: cotRefIX_Im8),
    (size: 3; cycles: 5; op:coLD    ; dest: cotRefIY    ; src: cotRefIY_Im8),
    (size: 2; cycles: 5; op:coLD    ; dest: cotRefIY    ; src: cotRefIX_L  ),
    (size: 2; cycles: 5; op:coLD    ; dest: cotRefIY    ; src: cotRefIY_L  ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSLA   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSLA   ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coSLA   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coSLA   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSLL   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSLL   ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coSLL   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coSLL   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSRA   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSRA   ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coSRA   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coSRA   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSRL   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSRL   ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coSRL   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coSRL   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coRL    ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coRL    ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coRL    ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coRL    ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coRLC   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coRLC   ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coRLC   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coRLC   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coRR    ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coRR    ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coRR    ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coRR    ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coRRC   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coRRC   ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coRRC   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coRRC   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coCPL   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coCPL   ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coCPL   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coCPL   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coNEG   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coNEG   ; dest: cotB        ; src: cotNone     ),
    (size: 3; cycles: 5; op:coNEG   ; dest: cotRefBRIm8 ; src: cotNone     ),
    (size: 2; cycles: 4; op:coNEG   ; dest: cotRefHL    ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSEP   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 3; op:coHALT  ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 3; op:coSLP   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 2; op:coAND   ; dest: cotB        ; src: cotIm8      ),
    (size: 3; cycles: 2; op:coAND   ; dest: cotL        ; src: cotIm8      ),
    (size: 3; cycles: 2; op:coAND   ; dest: cotH        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 3; op:coOR    ; dest: cotB        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coOR    ; dest: cotL        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coOR    ; dest: cotH        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 3; op:coXOR   ; dest: cotB        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coXOR   ; dest: cotL        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coXOR   ; dest: cotH        ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 3; op:coCP    ; dest: cotB        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCP    ; dest: cotL        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCP    ; dest: cotH        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCP    ; dest: cotBR       ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotA        ; src: cotBR       ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotA        ; src: cotSC       ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotBR       ; src: cotA        ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotSC       ; src: cotA        ),
    (size: 3; cycles: 4; op:coLD    ; dest: cotNB       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coLD    ; dest: cotEP       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coLD    ; dest: cotXP       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coLD    ; dest: cotYP       ; src: cotIm8      ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotA        ; src: cotNB       ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotA        ; src: cotEP       ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotA        ; src: cotXP       ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotA        ; src: cotYP       ),
    (size: 2; cycles: 3; op:coLD    ; dest: cotNB       ; src: cotA        ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotEP       ; src: cotA        ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotXP       ; src: cotA        ),
    (size: 2; cycles: 2; op:coLD    ; dest: cotYP       ; src: cotA        ),
    (size: 4; cycles: 5; op:coLD    ; dest: cotA        ; src: cotRefIm16  ),
    (size: 4; cycles: 5; op:coLD    ; dest: cotB        ; src: cotRefIm16  ),
    (size: 4; cycles: 5; op:coLD    ; dest: cotL        ; src: cotRefIm16  ),
    (size: 4; cycles: 5; op:coLD    ; dest: cotH        ; src: cotRefIm16  ),
    (size: 4; cycles: 5; op:coLD    ; dest: cotRefIm16  ; src: cotA        ),
    (size: 4; cycles: 5; op:coLD    ; dest: cotRefIm16  ; src: cotB        ),
    (size: 4; cycles: 5; op:coLD    ; dest: cotRefIm16  ; src: cotL        ),
    (size: 4; cycles: 5; op:coLD    ; dest: cotRefIm16  ; src: cotH        ),
    (size: 2; cycles:12; op:coMLT   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles:13; op:coDIV   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotLT       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotLE       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotGT       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotGE       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotV        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotNV       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotP        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotM        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotF0       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotF1       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotF2       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotF3       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotNF0      ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotNF1      ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotNF2      ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coJRS   ; dest: cotNF3      ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotLT       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotLE       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotGT       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotGE       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotV        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotNV       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotP        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotM        ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotF0       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotF1       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotF2       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotF3       ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotNF0      ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotNF1      ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotNF2      ; src: cotIm8      ),
    (size: 3; cycles: 3; op:coCARS  ; dest: cotNF3      ; src: cotIm8      ),

    // $CF expanded opcodes
    (size: 2; cycles: 4; op:coADDw  ; dest: cotBA       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotBA       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotBA       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotBA       ; src: cotIY       ),
    (size: 2; cycles: 4; op:coADCw  ; dest: cotBA       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coADCw  ; dest: cotBA       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coADCw  ; dest: cotBA       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coADCw  ; dest: cotBA       ; src: cotIY       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotBA       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotBA       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotBA       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotBA       ; src: cotIY       ),
    (size: 2; cycles: 4; op:coSBCw  ; dest: cotBA       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coSBCw  ; dest: cotBA       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coSBCw  ; dest: cotBA       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coSBCw  ; dest: cotBA       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotBA       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotBA       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotBA       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotBA       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotHL       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotHL       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotHL       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotHL       ; src: cotIY       ),
    (size: 2; cycles: 4; op:coADCw  ; dest: cotHL       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coADCw  ; dest: cotHL       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coADCw  ; dest: cotHL       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coADCw  ; dest: cotHL       ; src: cotIY       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotHL       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotHL       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotHL       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotHL       ; src: cotIY       ),
    (size: 2; cycles: 4; op:coSBCw  ; dest: cotHL       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coSBCw  ; dest: cotHL       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coSBCw  ; dest: cotHL       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coSBCw  ; dest: cotHL       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotHL       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotHL       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotHL       ; src: cotIX       ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotHL       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotIX       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotIX       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotIY       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotIY       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotSP       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coADDw  ; dest: cotSP       ; src: cotHL       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotIX       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotIX       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotIY       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotIY       ; src: cotHL       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotSP       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coSUBw  ; dest: cotSP       ; src: cotHL       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotSP       ; src: cotBA       ),
    (size: 2; cycles: 4; op:coCPw   ; dest: cotSP       ; src: cotHL       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 4; cycles: 4; op:coADCw  ; dest: cotBA       ; src: cotIm16     ),
    (size: 4; cycles: 4; op:coADCw  ; dest: cotHL       ; src: cotIm16     ),
    (size: 4; cycles: 4; op:coSBCw  ; dest: cotBA       ; src: cotIm16     ),
    (size: 4; cycles: 4; op:coSBCw  ; dest: cotHL       ; src: cotIm16     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 4; cycles: 4; op:coADDw  ; dest: cotSP       ; src: cotIm16     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 4; cycles: 4; op:coSUBw  ; dest: cotSP       ; src: cotIm16     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 4; cycles: 4; op:coCPw   ; dest: cotSP       ; src: cotIm16     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 4; cycles: 4; op:coLDw   ; dest: cotSP       ; src: cotIm16     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 3; cycles: 6; op:coLDw   ; dest: cotBA       ; src: cotRefSP_Im8),
    (size: 3; cycles: 6; op:coLDw   ; dest: cotHL       ; src: cotRefSP_Im8),
    (size: 3; cycles: 6; op:coLDw   ; dest: cotIX       ; src: cotRefSP_Im8),
    (size: 3; cycles: 6; op:coLDw   ; dest: cotIY       ; src: cotRefSP_Im8),
    (size: 3; cycles: 6; op:coLDw   ; dest: cotRefSP_Im8; src: cotBA       ),
    (size: 3; cycles: 6; op:coLDw   ; dest: cotRefSP_Im8; src: cotHL       ),
    (size: 3; cycles: 6; op:coLDw   ; dest: cotRefSP_Im8; src: cotIX       ),
    (size: 3; cycles: 6; op:coLDw   ; dest: cotRefSP_Im8; src: cotIY       ),
    (size: 4; cycles: 4; op:coLDw   ; dest: cotSP       ; src: cotRefIm16  ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 4; cycles: 6; op:coLDw   ; dest: cotRefIm16  ; src: cotSP       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 3; op:coPUSH  ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coPUSH  ; dest: cotB        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coPUSH  ; dest: cotL        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coPUSH  ; dest: cotH        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coPOP   ; dest: cotA        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coPOP   ; dest: cotB        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coPOP   ; dest: cotL        ; src: cotNone     ),
    (size: 2; cycles: 3; op:coPOP   ; dest: cotH        ; src: cotNone     ),
    (size: 2; cycles:12;op:coPUSH_ALL; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles:15;op:coPUSH_ALE; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles:11; op:coPOP_ALL; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles:14; op:coPOP_ALE; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotBA       ; src: cotRefHL    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotHL       ; src: cotRefHL    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotIX       ; src: cotRefHL    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotIY       ; src: cotRefHL    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefHL    ; src: cotBA       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefHL    ; src: cotHL       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefHL    ; src: cotIX       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefHL    ; src: cotIY       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotBA       ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotHL       ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotIX       ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotIY       ; src: cotRefIX    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefIX    ; src: cotBA       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefIX    ; src: cotHL       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefIX    ; src: cotIX       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefIX    ; src: cotIY       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotBA       ; src: cotRefIY    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotHL       ; src: cotRefIY    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotIX       ; src: cotRefIY    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotIY       ; src: cotRefIY    ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefIY    ; src: cotBA       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefIY    ; src: cotHL       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefIY    ; src: cotIX       ),
    (size: 2; cycles: 5; op:coLDw   ; dest: cotRefIY    ; src: cotIY       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotBA       ; src: cotBA       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotBA       ; src: cotHL       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotBA       ; src: cotIX       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotBA       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotHL       ; src: cotBA       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotHL       ; src: cotHL       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotHL       ; src: cotIX       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotHL       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIX       ; src: cotBA       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIX       ; src: cotHL       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIX       ; src: cotIX       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIX       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIY       ; src: cotBA       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIY       ; src: cotHL       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIY       ; src: cotIX       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIY       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotSP       ; src: cotBA       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotSP       ; src: cotHL       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotSP       ; src: cotIX       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotSP       ; src: cotIY       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotHL       ; src: cotSP       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotHL       ; src: cotPC       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotBA       ; src: cotSP       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotBA       ; src: cotPC       ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIX       ; src: cotSP       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     ),
    (size: 2; cycles: 2; op:coLDw   ; dest: cotIY       ; src: cotSP       ),
    (size: 2; cycles: 2; op:coBAD   ; dest: cotNone     ; src: cotNone     )
  );
  mem : Array [0..$FFFF] of byte;

procedure InitMicroCode;

function MicroCodeHumanReadable(var data: TCPUMicroCode): string;
function MicroCodeOperandsHumanReadable(var data: TCPUMicroCode; PC: integer): string;

function ReadByte(addr: integer): integer;
function ReadWord(addr: integer): integer;




implementation

uses Dialogs, SysUtils, main;

//-- Read one byte in memory --
function ReadByte(addr: integer): integer;
var i: integer;
begin
  if (addr<0) or (addr>=$10000) then
    raise Exception.Create('Invalid address requested for write ('+inttohex(addr,8)+')');

  Result:=mem[addr];
end;

//-- Read two bytes in memory --
function ReadWord(addr: integer): integer;
begin
  result := ReadByte(addr) + ReadByte(addr+1)*256;
end;


function MicroCodeHumanReadable(var data: TCPUMicroCode): string;
begin
  result := CPUOperationNames[data.op];
  if data.dest<>cotNone then
    result := result + ' ' + CPUOperandNames[data.dest];
  if data.src<>cotNone then
    result := result + ', ' + CPUOperandNames[data.src];
end;


function GetOperandText(op: TCPUOperation; opnd: TCPUOperand; addr, opnd_off: integer): string;
var s, rep: string;
    p, offset, size: integer;
begin
  result := '';
  if opnd=cotNone then exit;

  // this is just to suppress a warning
  offset := 0;
  size := 0;

  s := CPUOperandNames[opnd];
  p := Pos('nnnn', s);
  if p<>0 then begin
    offset := ReadWord(addr + opnd_off);
    size := 2;
  end else begin
    p := Pos('nn', s);
    if p<>0 then begin
      offset := ReadByte(addr + opnd_off);
      size := 1;
    end
  end;

  if p=0 then begin
    result:=s;
    exit;
  end;

  if (op in [coJRS, coJRL, coJP, coDJR, coCARS, coCARL]) or
     (opnd in [cotRefIm16]) then begin

    if not (opnd in [cotRefIm16]) then begin
      if size=1 then offset := ShortInt(offset);
      if (op=coCARS) and (ReadByte(addr)=$CE) then
        Inc(offset);
      //+PDE
      if (op=coJRS) and (ReadByte(addr)=$CE) then
        Inc(offset);
      //-PDE
      offset := LongWord(addr + offset + size) and $FFFF;
    end;

    if AddrNames[offset]<>'' then
      rep := AddrNames[offset]
    else
      rep := inttohex(offset, 4)+'H'; //PDE

  end else begin
    rep := '#'+inttohex(offset, size*2)+'H';  //PDE
  end;

  s := copy(s,1,p-1) + rep + copy(s,p+size*2,1000);

  result := s;
end;


function MicroCodeOperandsHumanReadable(var data: TCPUMicroCode; PC: integer): string;
var op, opnd1, opnd2: string;
begin
  op := CPUOperationNames[data.op];
  opnd1 := GetOperandText(data.op, data.dest, PC, data.d_imm_off);
  opnd2 := GetOperandText(data.op, data.src, PC, data.s_imm_off);
  if opnd2<>'' then opnd2:=', '+opnd2;
  result := Format('%-4s %s%s',[op,opnd1,opnd2]);
end;


// this functions pre-calcs shadow fields in the microcode array
// to speed up runtime execution

procedure InitMicroCode;
var i, bytes: integer;
begin
  for i:=0 to High(MicroCodeData) do begin

    if i>$FF then bytes:=2 else bytes:=1;
    with MicroCodeData[i] do begin
      d_imm_off:=0;
      s_imm_off:=0;
      if dest in [cotIm16, cotRefIm16] then begin
        d_imm_off:=bytes;
        Inc(bytes,2);
      end;
      if dest in [cotRefIX_Im8, cotRefIY_Im8, cotRefSP_Im8, cotIm8, cotRefBRIm8] then begin
        d_imm_off:=bytes;
        Inc(bytes);
      end;
      if src in [cotIm16, cotRefIm16] then begin
        s_imm_off:=bytes;
        Inc(bytes,2);
      end;
      if src in [cotRefIX_Im8, cotRefIY_Im8, cotRefSP_Im8, cotIm8, cotRefBRIm8] then begin
        s_imm_off:=bytes;
        Inc(bytes);
      end;

      if bytes<>size then begin
        ShowMessage('Invalid size for instruction '+
           MicroCodeHumanReadable(MicroCodeData[i])+
           #13#10'size on array '+inttostr(size)+', calculated '+inttostr(bytes));
      end;
    end;
  end;
end;



end.
